import os
import sys
import json
import shutil
import subprocess
import requests

script_path = os.path.normpath(os.path.abspath(__file__))
script_dir = os.path.dirname(script_path)

username = os.environ['USERNAME']

os.chdir(script_dir)

cygpath_home = r'minicygwin\home\{}'.format(username)
if not os.path.exists(cygpath_home):
    os.makedirs(cygpath_home)


def run_cmd(cmd, onfail=None):
    if os.system(cmd) != 0:
        print "Command '{}' failed.".format(cmd)
        if onfail:
            onfail()
        sys.exit(1)


def cygcall_out(cmd):
    old_path = os.environ['PATH']
    os.environ['PATH'] = '/bin'
    r = subprocess.check_output('minicygwin\\bin\\' + cmd, shell=True)
    os.environ['PATH'] = old_path
    return r


def cygcall(cmd, onfail=None):
    old_path =  os.environ['PATH']
    os.environ['PATH'] = '/bin'
    run_cmd('minicygwin\\bin\\' + cmd, onfail)
    os.environ['PATH'] = old_path


def find_auth_key(upload_target):
    keypath_file = 'keypath_{}.txt'.format(upload_target)
    if os.path.isfile(keypath_file):
        key_file = open(keypath_file).read().strip()
    else:
        key_file = upload_target + '.key'

    if not os.path.isfile(key_file):
        print 'key file for "{}" not found'.format(upload_target)
        sys.exit(1)
    cygpath_key = cygcall_out('cygpath {}'.format(key_file)).rstrip()
    return key_file, cygpath_key


def check_keys(upload_target_names):
    for target in upload_target_names:
        key_file, cygpath_key = find_auth_key(target)
        owner, rights = cygcall_out('stat {key} --printf "%U %A"'.format(key=cygpath_key)).split(' ')
        acl_default = cygcall_out('getfacl -cd {key}'.format(key=cygpath_key)).strip()
        if acl_default != '':
            print "Fix ACL atributes for '{}'... ".format(key_file),
            cygcall('setfacl -k {key}'.format(key=cygpath_key))
            print 'ok'

        if owner != username:
            print "Fix owner atributes for '{}'... ".format(key_file),
            # cygcall('cygstart -w --hide --action=runas takeown /F {}'.format(key_file))
            cygcall('cygstart -w --hide --action=runas chown {0}:{0} {1}'.format(username, cygpath_key))
            print 'ok'
        if rights != '-rw-------':
            print "Fix file access rights for '{}'... ".format(key_file),
            cygcall('chmod 600 {}'.format(cygpath_key))
            print 'ok'


def upload_webpacks(camelot_url, buildnumber, upload_targets):
    check_keys(map(str, upload_targets.keys()))

    print 'Fetching webpacks list...'
    r = requests.get('{camelot_url}/download_build_file/{buildnumber}/build_info'.format(
        camelot_url=camelot_url,
        buildnumber=buildnumber,
    ))
    if r.status_code != 200:
        print 'error: {} {}'.format(r.status_code, r.reason)
        sys.exit(1)
    build_info = json.loads(r.content)
    webpacks_list = build_info.get('additional_files', {}).get('webpacks', {})

    webpacks_count = len(webpacks_list)
    print '{} webpack(s) found.'.format(webpacks_count),
    if webpacks_count > 0:
        print map(str, webpacks_list.keys())
    else:
        print ''
        sys.exit(0)

    if os.path.exists('webpack'):
        print 'Removing old webpack...'
        shutil.rmtree('webpack')

    for webpack_name in webpacks_list.keys():
        url = '{camelot_url}/download_build_file/{buildnumber}/webpacks.{pack}'.format(
            camelot_url=camelot_url,
            pack=webpack_name,
            buildnumber=buildnumber)
        print 'Downloading {}'.format(url)
        zip_name = 'webpack-{}.zip'.format(webpack_name)
        run_cmd('curl -o {} -k {}'.format(zip_name, url))
        print 'Unzipping webpack'
        run_cmd('7za -owebpack x -aos {}'.format(zip_name))
        os.remove(zip_name)

    for cdn_name, [remote, port] in upload_targets.iteritems():
        print ''
        print "Upload to '{}'".format(cdn_name)
        key_file, cygpath_key = find_auth_key(cdn_name)
        print 'Used key:', key_file
        ssh_part = "ssh -o 'StrictHostKeyChecking no' -p {} -i {}".format(port, cygpath_key)
        rsync_options = r'-e \"{ssh_part}\" -r --chmod=Du=rwx,Dgo=rx,Fu=rw,Fgo=r'.format(ssh_part=ssh_part)
        cmd = 'bash --login -c "rsync {rsync_options} webpack/ {remote}"'.format(
            rsync_options=rsync_options,
            remote=remote,
        )
        print cmd
        cygcall(cmd)
        print "Upload to '{}' completed.".format(cdn_name)

    print ''
    print 'Cleanup... ',
    shutil.rmtree('webpack')
    print 'ok'
    print 'UPLOAD FINISHED SUCCESSFULLY'


def compile_self():
    run_cmd('pyinstaller --onefile {}'.format(script_path))
    shutil.move(os.path.join('dist', 'uploader.exe'), 'uploader.exe')
    shutil.rmtree('build')
    shutil.rmtree('dist')
    os.remove('uploader.spec')


def main():
    if len(sys.argv) != 2:
        print 'args error: specify build number parameter.'
        sys.exit(1)

    settings = json.load(open(os.path.join(script_dir, 'uploader_settings.json')))
    camelot_url = settings['camelot_url']
    upload_targets = settings['remotes']

    if sys.argv[1] == '--compile-self':
        compile_self()
    else:
        print 'Webpack Uploader 2.0'
        buildnumber = sys.argv[1]
        upload_webpacks(camelot_url, buildnumber, upload_targets)

if __name__ == '__main__':
    main()
